'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var reportWeakType = function reportWeakType(context, weakType) {
  return function (node) {
    context.report({
      data: { weakType: weakType },
      message: 'Unexpected use of weak type "{{weakType}}"',
      node: node
    });
  };
};

var genericTypeEvaluator = function genericTypeEvaluator(context) {
  var weakTypes = ['Function', 'Object'];

  return function (node) {
    var name = _lodash2.default.get(node, 'id.name');
    var isWeakType = weakTypes.indexOf(name) >= 0;

    if (isWeakType) {
      reportWeakType(context, name)(node);
    }
  };
};

exports.default = function (context) {
  return {
    AnyTypeAnnotation: reportWeakType(context, 'any'),
    GenericTypeAnnotation: genericTypeEvaluator(context)
  };
};

module.exports = exports['default'];